/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Fabian 'x3n' Landau
 *   Co-authors:
 *      Simon Miescher
 *
 */

/*
*
*
* An asteroid which can be destroyed. Some smaller asteroids are created and a pickup 
* spawns. 
*
*
*

*/


#include "Pawn.h"
#include "AsteroidMinable.h"


#include <algorithm>

#include "core/CoreIncludes.h"
#include "core/GameMode.h"
#include "core/XMLPort.h"
#include "core/EventIncludes.h"
#include "network/NetworkFunction.h"

#include "infos/PlayerInfo.h"
#include "controllers/Controller.h"
#include "gametypes/Gametype.h"
#include "graphics/ParticleSpawner.h"
#include "worldentities/ExplosionChunk.h"
#include "worldentities/ExplosionPart.h"
#include "weaponsystem/WeaponSystem.h"
#include "weaponsystem/WeaponSlot.h"
#include "weaponsystem/WeaponPack.h"
#include "weaponsystem/WeaponSet.h"
#include "weaponsystem/Munition.h"
#include "sound/WorldSound.h"
#include "core/object/ObjectListIterator.h"

#include "controllers/FormationController.h"

namespace orxonox
{
    RegisterClass(AsteroidMinable);

    AsteroidMinable::AsteroidMinable(Context* context) : Pawn(context)
    {
        RegisterObject(AsteroidMinable);

        this->bAlive_ = true;
        this->bVulnerable_ = true;

        this->health_ = 0;
        this->maxHealth_ = 0;
        this->initialHealth_ = 0;

        this->damageMultiplier_ = 1;

        this->spawnparticleduration_ = 3.0f;

        this->aimPosition_ = Vector3::ZERO;

        this->setRadarObjectColour(ColourValue::Yellow);
        this->setRadarObjectShape(RadarViewable::Shape::Dot);
 
    }

    AsteroidMinable::~AsteroidMinable()
    {

    }

    void AsteroidMinable::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(AsteroidMinable, XMLPort, xmlelement, mode);
    }

    void AsteroidMinable::XMLEventPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(AsteroidMinable, XMLEventPort, xmlelement, mode);

        XMLPortEventState(AsteroidMinable, BaseObject, "vulnerability", setVulnerable, xmlelement, mode);
    }

    void AsteroidMinable::registerVariables()
    {
        registerVariable(this->bAlive_,            VariableDirection::ToClient);
        registerVariable(this->bVulnerable_,       VariableDirection::ToClient);
        registerVariable(this->health_,            VariableDirection::ToClient);
        registerVariable(this->maxHealth_,         VariableDirection::ToClient);
    }

    void AsteroidMinable::tick(float dt)
    {
        SUPER(Pawn, tick, dt);

    }

    void AsteroidMinable::death() //ueberschreiben
    {
        this->setHealth(1);
        if (this->getGametype() && this->getGametype()->allowPawnDeath(this, this->lastHitOriginator_))
        {
            // Set bAlive_ to false and wait for destroyLater() to do the destruction
            this->bAlive_ = false;
            this->destroyLater();

            this->setDestroyWhenPlayerLeft(false);

        }
    }

}



/*
    void DronePickup::changedUsed(void)
    {
        SUPER(DronePickup, changedUsed);

        // If the pickup has transited to used.
        if(this->isUsed())
        {

                Pawn* pawn = this->carrierToPawnHelper();
                if(pawn == nullptr) // If the PickupCarrier is no Pawn, then this pickup is useless and therefore is destroyed.
                    this->Pickupable::destroy();

                //Attach to pawn
                Drone* drone = new Drone(pawn->getContext()); // this is neccessary because the projectiles fired need a valid creator for the particlespawner (when colliding against something)
                drone->addTemplate(this->getDroneTemplate());

                Controller* controller = drone->getController();
                DroneController* droneController = orxonox_cast<DroneController*>(controller);
                if(droneController != nullptr)
                {
                    droneController->setOwner(pawn);
                }

                Vector3 spawnPosition = pawn->getWorldPosition() + Vector3(30,0,-30);
                drone->setPosition(spawnPosition);

                // The pickup has been used up.
                this->setUsed(false);
        }
        else
        {
            // If either the pickup can only be used once or it is continuous and used up, it is destroyed upon setting it to unused.
            if(this->isOnce() || (this->isContinuous() ))
            {
                this->Pickupable::destroy();
            }
        }
    }
*/